# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Datacenter Widgets

"""

import pyVmomi

import pvc.widget.alarm
import pvc.widget.common
import pvc.widget.cluster
import pvc.widget.debug
import pvc.widget.event
import pvc.widget.hostsystem
import pvc.widget.menu
import pvc.widget.performance
import pvc.widget.virtualmachine

__all__ = [
    'DatacenterWidget', 'DatacenterActionWidget',
    'DatacenterClusterWidget', 'DatacenterHostSystemWidget',
    'DatacenterDatastoreWidget', 'DatacenterNetworkWidget',
    'DatacenterVirtualMachineWidget',
]


class DatacenterWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Datacenter Widget

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.Datacenter): A vim.Datacenter managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Summary',
                description='General information',
                on_select=self.summary
            ),
            pvc.widget.menu.MenuItem(
                tag='Actions',
                description='Available Action',
                on_select=DatacenterActionWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Configuration',
                description='Datacenter configuration'
            ),
            pvc.widget.menu.MenuItem(
                tag='Clusters',
                description='Manage clusters in datacenter',
                on_select=DatacenterClusterWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Hosts',
                description='Manage hosts in datacenter',
                on_select=DatacenterHostSystemWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Virtual Machines',
                description='Manage Virtual Machines in datacenter',
                on_select=DatacenterVirtualMachineWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Datastore',
                description='Manage datastores in datacenter',
                on_select=DatacenterDatastoreWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Networking',
                description='Manage networking in datacenter',
                on_select=DatacenterNetworkWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Performance',
                description='Performance Metrics',
                on_select=pvc.widget.performance.PerformanceProviderWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Events',
                description='View Events',
                on_select=pvc.widget.event.EventWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Alarms',
                description='View triggered alarms',
                on_select=pvc.widget.common.alarm_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Debug',
                description='Start a Python REPL console',
                on_select=pvc.widget.debug.DebugWidget,
                on_select_args=(locals(), globals())
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select item from menu'
        )

        menu.display()

    def summary(self):
        """
        Datacenter general information

        """
        self.dialog.infobox(
            title=self.title,
            text='Retrieving information ...'
        )

        elements = [
            pvc.widget.form.FormElement(
                label='Overall Status',
                item=self.obj.overallStatus
            ),
        ]

        form = pvc.widget.form.Form(
            dialog=self.dialog,
            form_elements=elements,
            title=self.title,
            text='Datacenter general information'
        )

        form.display()


class DatacenterActionWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Datacenter Actions Widget

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.Datacenter): A vim.Datacenter managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Rename',
                description='Rename datacenter',
                on_select=pvc.widget.common.rename,
                on_select_args=(self.obj, self.dialog)
            ),
            pvc.widget.menu.MenuItem(
                tag='Remove',
                description='Remove datacenter',
                on_select=pvc.widget.common.remove,
                on_select_args=(self.obj, self.dialog)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an action to be performed'
        )

        menu.display()


class DatacenterClusterWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Datacenter Cluster Widget

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.Datacenter): A vim.Datacenter managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Create',
                description='Create new cluster in datacenter',
                on_select=self.create_cluster
            ),
            pvc.widget.menu.MenuItem(
                tag='Remove',
                description='Remove cluster from datacenter'
            ),
            pvc.widget.menu.MenuItem(
                tag='View',
                description='View clusters in datacenter',
                on_select=pvc.widget.common.cluster_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an action to be performed'
        )

        menu.display()

    def create_cluster(self):
        """
        Create a new cluster in the datacenter

        """
        code, name = self.dialog.inputbox(
            title=self.title,
            text='Provide name for the cluster to be created'
        )

        if code in (self.dialog.CANCEL, self.dialog.ESC):
            return

        if not name:
            self.dialog.msgbox(
                title=self.title,
                text='Error, invalid input provided'
            )
            return

        self.dialog.infobox(
            title=self.title,
            text='Creating cluster {} ...'.format(name)
        )

        try:
            self.obj.hostFolder.CreateClusterEx(
                name=name,
                spec=pyVmomi.vim.cluster.ConfigSpecEx()
            )
        except pyVmomi.vim.MethodFault as e:
            self.dialog.msgbox(
                title=self.title,
                text=e.msg
            )


class DatacenterHostSystemWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Widget for managing hosts in a datacenter

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.Datacenter): A vim.Datacenter managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='View',
                description='View hosts in datacenter',
                on_select=self.host_menu
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select item from menu'
        )

        menu.display()

    def host_menu(self):
        self.dialog.infobox(
            title=self.title,
            text='Retrieving information ...'
        )

        view = self.agent.get_container_view(
            obj_type=[pyVmomi.vim.HostSystem],
            container=self.obj
        )
        properties = self.agent.collect_properties(
            view_ref=view,
            obj_type=pyVmomi.vim.HostSystem,
            path_set=['name', 'runtime.connectionState'],
            include_mors=True
        )
        view.DestroyView()

        if not properties:
            self.dialog.msgbox(
                title=self.title,
                text='No hosts found in datacenter'
            )
            return

        items = [
            pvc.widget.menu.MenuItem(
                tag=host['name'],
                description=host['runtime.connectionState'],
                on_select=pvc.widget.hostsystem.HostSystemWidget,
                on_select_args=(self.agent, self.dialog, host['obj'])
            ) for host in properties
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select a host from the menu'
        )

        menu.display()


class DatacenterDatastoreWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Widget for managing datastores in a datacenter

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.Datacenter): A vim.Datacenter managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='View',
                description='View datastores in datacenter',
                on_select=pvc.widget.common.datastore_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select item from menu'
        )

        menu.display()


class DatacenterNetworkWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Widget for managing networking in a datacenter

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.Datacenter): A vim.Datacenter managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='View',
                description='View networks in datacenter',
                on_select=pvc.widget.common.network_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select item from menu'
        )

        menu.display()


class DatacenterVirtualMachineWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Widget for managing virtual machines in a datacenter

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.Datacenter): A vim.Datacenter managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Create',
                description='Create new Virtual Machine',
                on_select=pvc.widget.virtualmachine.CreateVirtualMachineWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='View',
                description='Virtual Machines in datacenter',
                on_select=self.virtual_machine_menu
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an action to be performed'
        )

        menu.display()

    def virtual_machine_menu(self):
        self.dialog.infobox(
            title=self.title,
            text='Retrieving information ...'
        )

        view = self.agent.get_container_view(
            obj_type=[pyVmomi.vim.VirtualMachine],
            container=self.obj
        )
        properties = self.agent.collect_properties(
            view_ref=view,
            obj_type=pyVmomi.vim.VirtualMachine,
            path_set=['name', 'runtime.powerState'],
            include_mors=True
        )
        view.DestroyView()

        if not properties:
            self.dialog.msgbox(
                title=self.title,
                text='No virtual machines found in datacenter'
            )
            return

        items = [
            pvc.widget.menu.MenuItem(
                tag=vm['name'],
                description=vm['runtime.powerState'],
                on_select=pvc.widget.virtualmachine.VirtualMachineWidget,
                on_select_args=(self.agent, self.dialog, vm['obj'])
            ) for vm in properties
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select a virtual machine from the menu'
        )

        menu.display()



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Gauge Widgets

"""

import pyVmomi

from time import sleep

__all__ = ['TaskGauge']


class TaskGauge(object):
    def __init__(self, dialog, task, interval=0.5, **kwargs):
        """
        A gauge for displaying progress of a task

        Args:
           dialog   (dialog.Dialog): A Dialog instance
           task          (vim.Task): A Task instance
           interval         (float): Check task state each 'interval' seconds
           kwargs            (dict): Additional args to be passed to dialog(1)

        """
        self.dialog = dialog
        self.task = task
        self.interval = interval
        self.kwargs = kwargs

    def display(self):
        """
        docstring

        """
        self.dialog.gauge_start(
            **self.kwargs
        )

        while True:
            if self.task.info.state in (pyVmomi.vim.TaskInfoState.queued, pyVmomi.vim.TaskInfoState.running):
                progress = self.task.info.progress if self.task.info.progress else 0
                self.dialog.gauge_update(progress)
            elif self.task.info.state == pyVmomi.vim.TaskInfoState.success:
                break
            elif self.task.info.state == pyVmomi.vim.TaskInfoState.error:
                self.dialog.msgbox(
                    title='Task Error',
                    text=self.task.info.error.msg
                )
                break
            sleep(self.interval)

        self.dialog.gauge_stop()



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Common Widgets Module

"""

import pyVmomi

import pvc.widget.alarm
import pvc.widget.cluster
import pvc.widget.datacenter
import pvc.widget.menu
import pvc.widget.gauge
import pvc.widget.datastore
import pvc.widget.network
import pvc.widget.hostsystem
import pvc.widget.session
import pvc.widget.radiolist
import pvc.widget.virtualmachine

__all__ = [
    'rename', 'network_menu', 'datastore_menu',
    'host_menu', 'hostmount_menu', 'virtual_machine_menu',
    'session_menu', 'alarm_menu', 'choose_folder',
    'choose_datacenter', 'choose_cluster', 'choose_datastore',
    'inventory_search_by_dns', 'inventory_search_by_ip',
    'inventory_search_by_uuid', 'datacenter_menu', 'remove',
    'choose_network', 'host_service_menu',
]


def rename(obj, dialog):
    """
    Rename a Managed Entity

    Args:
        obj    (vim.ManagedEntity): A Managed Entity
        dialog     (dialog.Dialog): A Dialog instance

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    code, new_name = dialog.inputbox(
        title=title,
        text='New name for {}?'.format(obj.name),
        init=obj.name
    )

    if code in (dialog.CANCEL, dialog.ESC):
        return

    task = obj.Rename(newName=new_name)
    gauge = pvc.widget.gauge.TaskGauge(
        dialog=dialog,
        task=task,
        title=title,
        text='Renaming {} to {} ...'.format(obj.name, new_name)
    )

    gauge.display()


def remove(obj, dialog):
    """
    Remove a Managed Entity

    NOTE: This operation also removes any child objects of the
          managed entity.

    Args:
        obj    (vim.ManagedEntity): A Managed Entity
        dialog     (dialog.Dialog): A Dialog instance

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    code = dialog.yesno(
        title=title,
        text='Remove {} and all child objects?'.format(obj.name)
    )

    if code in (dialog.CANCEL, dialog.ESC):
        return

    task = obj.Destroy()
    gauge = pvc.widget.gauge.TaskGauge(
        dialog=dialog,
        task=task,
        title=title,
        text='Removing {} and any child objects ...'.format(obj.name)
    )

    gauge.display()


def datacenter_menu(agent, dialog, folder=None):
    """
    A widget to display a menu of Datacenter entities

    Args:
        agent         (VConnector): A VConnector instance
        dialog     (dialog.Dailog): A Dialog instance
        folder        (vim.Folder): A vim.Folder entity

    """
    dialog.infobox(
        text='Retrieving information ...'
    )

    if not folder:
        folder = agent.si.content.rootFolder

    view = agent.get_container_view(
        obj_type=[pyVmomi.vim.Datacenter],
        container=folder
    )
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.Datacenter,
        path_set=['name', 'overallStatus'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        dialog.msgbox(
            title='No objects found',
            text='No datacenters managed entities found'
        )
        return

    items = [
        pvc.widget.menu.MenuItem(
            tag=dc['name'],
            description=dc['overallStatus'],
            on_select=pvc.widget.datacenter.DatacenterWidget,
            on_select_args=(agent, dialog, dc['obj'])
        ) for dc in properties
    ]

    menu = pvc.widget.menu.Menu(
        items=items,
        dialog=dialog,
        title='Select Datacenter',
        text=''
    )

    menu.display()


def cluster_menu(agent, dialog, folder=None):
    """
    A widget to display a menu of vim.ClusterComputeResource entities

    Args:
        agent     (VConnector): A VConnector instance
        dialog (dialog.Dailog): A Dialog instance
        folder    (vim.Folder): A vim.Folder entity

    """
    dialog.infobox(
        text='Retrieving information ...'
    )

    if not folder:
        folder = agent.si.content.rootFolder

    view = agent.get_container_view(
        obj_type=[pyVmomi.vim.ClusterComputeResource],
        container=folder
    )
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.ClusterComputeResource,
        path_set=['name', 'overallStatus'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        dialog.msgbox(
            title='No objects found',
            text='No cluster managed entities found'
        )
        return

    items = [
        pvc.widget.menu.MenuItem(
            tag=cluster['name'],
            description=cluster['overallStatus'],
            on_select=pvc.widget.cluster.ClusterWidget,
            on_select_args=(agent, dialog, cluster['obj'])
        ) for cluster in properties
    ]

    menu = pvc.widget.menu.Menu(
        items=items,
        dialog=dialog,
        title='Select Cluster',
        text=''
    )

    menu.display()


def host_menu(agent, dialog, obj):
    """
    A widget to display a menu of HostSystem entities

    Args:
        agent         (VConnector): A VConnector instance
        dialog     (dialog.Dailog): A Dialog instance
        obj    (vim.ManagedEntity): A Managed Entity

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    dialog.infobox(
        title=title,
        text='Retrieving information ...'
    )

    if not hasattr(obj, 'host'):
        dialog.msgbox(
            title=title,
            text='Entity does not contain a host property'
        )
        return

    view = agent.get_list_view(obj.host)
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.HostSystem,
        path_set=['name', 'runtime.connectionState'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        dialog.msgbox(
            title=title,
            text='No hosts found on this managed entity'
        )
        return

    items = [
        pvc.widget.menu.MenuItem(
            tag=host['name'],
            description=host['runtime.connectionState'],
            on_select=pvc.widget.hostsystem.HostSystemWidget,
            on_select_args=(agent, dialog, host['obj'])
        ) for host in properties
    ]

    menu = pvc.widget.menu.Menu(
        items=items,
        dialog=dialog,
        title=title,
        text=''
    )

    menu.display()


def hostmount_menu(agent, dialog, obj):
    """
    A widget to display a menu of DatastoreHostMount instances

    To be used with datastores in order to provide a menu of
    hosts mounting a particular datastore

    Args:
        agent     (VConnector): A VConnector instance
        dialog (dialog.Dailog): A Dialog instance
        obj    (vim.Datastore): A Managed Entity

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    dialog.infobox(
        title=title,
        text='Retrieving information ...'
    )

    if not hasattr(obj, 'host'):
        dialog.msgbox(
            title=title,
            text='Entity does not contain a host property'
        )
        return

    hosts = [h.key for h in obj.host]
    view = agent.get_list_view(hosts)
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.HostSystem,
        path_set=['name', 'runtime.connectionState'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        dialog.msgbox(
            title=title,
            text='No hosts have mounted the datastore'
        )
        return

    items = [
        pvc.widget.menu.MenuItem(
            tag=host['name'],
            description=host['runtime.connectionState'],
            on_select=pvc.widget.hostsystem.HostSystemWidget,
            on_select_args=(agent, dialog, host['obj'])
        ) for host in properties
    ]

    menu = pvc.widget.menu.Menu(
        items=items,
        dialog=dialog,
        title=title,
        text='',
    )

    menu.display()


def host_service_menu(agent, dialog, obj):
    """
    A widget to display a menu of services in a HostSystem

    Args:
        agent      (VConnector): A VConnector instance
        dialog  (dialog.Dailog): A Dialog instance
        obj    (vim.HostSystem): A HostSystem entity

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    dialog.infobox(
        title=title,
        text='Retrieving information ...'
    )

    items = [
        pvc.widget.menu.MenuItem(
            tag=service.key,
            description=service.label,
            on_select=pvc.widget.hostsystem.HostSystemServiceWidget,
            on_select_args=(agent, dialog, obj, service)
        ) for service in obj.config.service.service
    ]

    menu = pvc.widget.menu.Menu(
        items=items,
        dialog=dialog,
        title=title,
        text='Choose a service to manage'
    )

    menu.display()


def network_menu(agent, dialog, obj):
    """
    A widget to display the networks by a Managed Entity

    Args:
        agent         (VConnector): A VConnector instance
        dialog     (dialog.Dailog): A Dialog instance
        obj    (vim.ManagedEntity): A Managed Entity

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    dialog.infobox(
        title=title,
        text='Retrieving information ...'
    )

    if not hasattr(obj, 'network'):
        dialog.msgbox(
            title=title,
            text='Entity does not contain a network property'
        )
        return

    view = agent.get_list_view(obj.network)
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.Network,
        path_set=['name', 'summary.accessible'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        dialog.msgbox(
            title=title,
            text='No networks found for this managed entity'
        )
        return

    items = [
        pvc.widget.menu.MenuItem(
            tag=network['name'],
            description='Accessible' if network['summary.accessible'] else 'Not Accessible',
            on_select=pvc.widget.network.NetworkWidget,
            on_select_args=(agent, dialog, network['obj'])
        ) for network in properties
    ]

    menu = pvc.widget.menu.Menu(
        items=items,
        dialog=dialog,
        title=title,
        text=''
    )

    menu.display()


def virtual_machine_menu(agent, dialog, obj):
    """
    A widget to display the VMs contained within a Managed Entity, e.g.
    HostSystem, ClusterComputeResource, Datastore, etc.

    Args:
        agent         (VConnector): A VConnector instance
        dialog     (dialog.Dailog): A Dialog instance
        obj    (vim.ManagedEntity): A Managed Entity

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    dialog.infobox(
        title=title,
        text='Retrieving information ...'
    )

    if not hasattr(obj, 'vm'):
        dialog.msgbox(
            title=title,
            text='Entity does not contain a vm property'
        )
        return

    view = agent.get_list_view(obj.vm)
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.VirtualMachine,
        path_set=['name', 'runtime.powerState'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        dialog.msgbox(
            title=title,
            text='No virtual machines found for this managed entity'
        )
        return

    items = [
        pvc.widget.menu.MenuItem(
            tag=vm['name'],
            description=vm['runtime.powerState'],
            on_select=pvc.widget.virtualmachine.VirtualMachineWidget,
            on_select_args=(agent, dialog, vm['obj'])
        ) for vm in properties
    ]

    menu = pvc.widget.menu.Menu(
        items=items,
        dialog=dialog,
        title=title,
        text=''
    )

    menu.display()


def datastore_menu(agent, dialog, obj):
    """
    A widget to get all Datastores used by a managed entity, e.g.
    HostSystem, VirtualMachine, Cluster, etc.

    Args:
        agent         (VConnector): A VConnector instance
        dialog     (dialog.Dailog): A Dialog instance
        obj    (vim.ManagedEntity): A Managed Entity

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    dialog.infobox(
        title=title,
        text='Retrieving information ...'
    )

    if not hasattr(obj, 'datastore'):
        dialog.msgbox(
            title=title,
            text='Entity does not contain a datastore property'
        )
        return

    view = agent.get_list_view(obj.datastore)
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.Datastore,
        path_set=['name', 'summary.accessible'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        dialog.msgbox(
            title=title,
            text='No datastores found for this managed entity'
        )
        return

    items = [
        pvc.widget.menu.MenuItem(
            tag=ds['name'],
            description='Accessible' if ds['summary.accessible'] else 'Not Accessible',
            on_select=pvc.widget.datastore.DatastoreWidget,
            on_select_args=(agent, dialog, ds['obj'])
        ) for ds in properties
    ]

    menu = pvc.widget.menu.Menu(
        items=items,
        dialog=dialog,
        title=title,
        text=''
    )

    menu.display()


def session_menu(agent, dialog):
    """
    A widget to display a menu of current sessions

    Args:
        agent         (VConnector): A VConnector instance
        dialog     (dialog.Dailog): A Dialog instance

    """
    dialog.infobox(
        title='Sessions',
        text='Retrieving information ...'
    )

    try:
        sm = agent.si.content.sessionManager
        session_list = sm.sessionList
    except pyVmomi.vim.NoPermission:
        dialog.msgbox(
            title='Access Denied',
            text='No permissions to view sessions'
        )
        return

    items = [
        pvc.widget.menu.MenuItem(
            tag=session.key,
            description='{}@{}'.format(session.userName, session.ipAddress),
            on_select=pvc.widget.session.SessionWidget,
            on_select_args=(agent, dialog, session)
        ) for session in session_list
    ]

    menu = pvc.widget.menu.Menu(
        items=items,
        dialog=dialog,
        title='Sessions',
        text='Select a session for more detais',
    )

    menu.display()


def alarm_menu(agent, dialog, obj):
    """
    Args:
        agent         (VConnector): A VConnector instance
        dialog     (dialog.Dailog): A Dialog instance
        obj    (vim.ManagedEntity): A Managed Entity

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    dialog.infobox(
        title=title,
        text='Retrieving information ...'
    )

    if not obj.triggeredAlarmState:
        dialog.msgbox(
            title=title,
            text='No triggered alarms'
        )
        return

    items = [
        pvc.widget.menu.MenuItem(
            tag=alarm.key,
            description=alarm.alarm.info.name,
            on_select=pvc.widget.alarm.AlarmWidget,
            on_select_args=(agent, dialog, alarm)
        ) for alarm in obj.triggeredAlarmState
    ]

    menu = pvc.widget.menu.Menu(
        items=items,
        dialog=dialog,
        title=title,
        text='Select an alarm for more details'
    )

    menu.display()


def choose_folder(agent, dialog):
    """
    Prompts the user to choose a folder

    Args:
        agent            (VConnector): A VConnector instance
        dialog        (dialog.Dialog): A Dialog instance

    Returns:
        A vim.Folder managed entity

    """
    dialog.infobox(
        text='Retrieving information ...'
    )

    view = agent.get_container_view(obj_type=[pyVmomi.vim.Folder])
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.Folder,
        path_set=['name'],
        include_mors=True
    )
    view.DestroyView()

    # Remove all occurrencies of 'vm', 'host', 'datastore' and
    # 'network' from the collected folders as these ones are
    # reserved and we cannot create a datacenter there
    folders = [f for f in properties if f['name'] not in ('vm', 'host', 'datastore', 'network')]

    if not folders:
        return agent.si.content.rootFolder

    items = [
        pvc.widget.radiolist.RadioListItem(tag=folder['name'])
        for folder in folders
    ]
    radiolist = pvc.widget.radiolist.RadioList(
        items=items,
        dialog=dialog,
        title='Choose Folder',
        text='Choose a folder or CANCEL to select the root folder',
    )

    code, tag = radiolist.display()

    if not tag:
        return agent.si.content.rootFolder

    return [f['obj'] for f in properties if f['name'] == tag].pop()


def choose_datacenter(agent, dialog, all_datacenters_option=False):
    """
    Prompts the user to choose a datacenter

    Convinience function that can be used to choose a datacenter,
    which result can be used for example to restrict search of
    inventory to a specific datacenter only, or be used for other
    purposes, e.g. choosing a datacenter where to deploy a Virtual Machine.

    Args:
        agent            (VConnector): A VConnector instance
        dialog        (dialog.Dialog): A Dialog instance
        all_datacenters_option (bool): If True then an option to choose all
                                       datacenter is provided as well

    Returns:
        A vim.Datacenter managed entity if a datacenter has been
        selected. Returns None if there are no datacenters found or
        if selected the 'All Datacenters' option.

    """
    dialog.infobox(
        text='Retrieving information ...'
    )

    view = agent.get_datacenter_view()
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.Datacenter,
        path_set=['name'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        return

    items = []
    if all_datacenters_option:
        items.append(
            pvc.widget.radiolist.RadioListItem(tag='All Datacenters')
        )

    datacenters = [
        pvc.widget.radiolist.RadioListItem(tag=datacenter['name'])
        for datacenter in properties
    ]
    items.extend(datacenters)

    radiolist = pvc.widget.radiolist.RadioList(
        items=items,
        dialog=dialog,
        title='Choose Datacenter',
        text='Choose a Datacenter from the list below'
    )

    code, tag = radiolist.display()

    if not tag:
        return
    elif all_datacenters_option and tag == 'All Datacenters':
        return

    return [d['obj'] for d in properties if d['name'] == tag].pop()


def choose_cluster(agent, dialog, folder=None):
    """
    Prompts the user to choose a cluster

    Args:
        agent     (VConnector): A VConnector instance
        dialog (dialog.Dailog): A Dialog instance
        folder    (vim.Folder): A vim.Folder entity

    Returns:
        A vim.ClusterComputeResource managed entity if the
        user chose a cluster.

        Returns None if no cluster has been
        selected or there are no clusters existing

    """
    dialog.infobox(
        text='Retrieving information ...'
    )

    if not folder:
        folder = agent.si.content.rootFolder

    view = agent.get_container_view(
        obj_type=[pyVmomi.vim.ClusterComputeResource],
        container=folder
    )

    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.ClusterComputeResource,
        path_set=['name', 'overallStatus'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        return

    items = [
        pvc.widget.radiolist.RadioListItem(
            tag=cluster['name'],
            description=cluster['overallStatus'],
        ) for cluster in properties
    ]

    radiolist = pvc.widget.radiolist.RadioList(
        items=items,
        dialog=dialog,
        title='Choose Cluster',
        text='Choose a cluster from the list below'
    )

    code, tag = radiolist.display()

    if code in (dialog.CANCEL, dialog.ESC) or not tag:
        return

    return [c['obj'] for c in properties if c['name'] == tag].pop()


def choose_host(agent, dialog, folder=None):
    """
    Prompts the user to choose a host

    Args:
        agent     (VConnector): A VConnector instance
        dialog (dialog.Dailog): A Dialog instance
        folder    (vim.Folder): A vim.Folder entity

    Returns:
        A vim.HostSystem managed entity if the
        user chose a host.

        Returns None if no host has been
        selected or there are no hosts existing

    """
    dialog.infobox(
        text='Retrieving information ...'
    )

    if not folder:
        folder = agent.si.content.rootFolder

    view = agent.get_container_view(
        obj_type=[pyVmomi.vim.HostSystem],
        container=folder
    )

    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.HostSystem,
        path_set=['name', 'runtime.connectionState'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        return

    items = [
        pvc.widget.radiolist.RadioListItem(
            tag=host['name'],
            description=host['runtime.connectionState'],
        ) for host in properties
    ]

    radiolist = pvc.widget.radiolist.RadioList(
        items=items,
        dialog=dialog,
        title='Choose Host',
        text='Choose a host from the list below'
    )

    code, tag = radiolist.display()

    if code in (dialog.CANCEL, dialog.ESC) or not tag:
        return

    return [h['obj'] for h in properties if h['name'] == tag].pop()


def choose_datastore(agent, dialog, obj):
    """
    Prompts the user to choose a datastore

    Args:
        agent         (VConnector): A VConnector instance
        dialog     (dialog.Dailog): A Dialog instance
        obj    (vim.ManagedEntity): A Managed Entity

    Returns:
        A vim.Datastore managed entity if a datastore has been
        chosen

        Return None if no datastore has been selected or
        there are no datastores existing

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    dialog.infobox(
        title=title,
        text='Retrieving information ...'
    )

    if not hasattr(obj, 'datastore'):
        return

    view = agent.get_list_view(obj.datastore)
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.Datastore,
        path_set=['name', 'summary.accessible'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        return

    items = [
        pvc.widget.radiolist.RadioListItem(
            tag=ds['name'],
            description='Accessible' if ds['summary.accessible'] else 'Not Accessible',
        ) for ds in properties
    ]

    radiolist = pvc.widget.radiolist.RadioList(
        items=items,
        dialog=dialog,
        title='Choose Datastore',
        text='Choose a datastore from the list below'
    )

    code, tag = radiolist.display()

    if code in (dialog.CANCEL, dialog.ESC) or not tag:
        return

    return [ds['obj'] for ds in properties if ds['name'] == tag].pop()


def choose_network(agent, dialog, obj):
    """
    Prompts the user to choose a network

    Args:
        agent         (VConnector): A VConnector instance
        dialog     (dialog.Dailog): A Dialog instance
        obj    (vim.ManagedEntity): A Managed Entity

    Returns:
        A vim.Network managed entity if a network has been
        chosen

        Return None if no network has been selected or
        there are no networks existing

    """
    title = '{} ({})'.format(obj.name, obj.__class__.__name__)

    dialog.infobox(
        title=title,
        text='Retrieving information ...'
    )

    if not hasattr(obj, 'network'):
        return

    view = agent.get_list_view(obj.network)
    properties = agent.collect_properties(
        view_ref=view,
        obj_type=pyVmomi.vim.Network,
        path_set=['name', 'summary.accessible'],
        include_mors=True
    )
    view.DestroyView()

    if not properties:
        return

    items = [
        pvc.widget.radiolist.RadioListItem(
            tag=network['name'],
            description='Accessible' if network['summary.accessible'] else 'Not Accessible',
        ) for network in properties
    ]

    radiolist = pvc.widget.radiolist.RadioList(
        items=items,
        dialog=dialog,
        title='Choose Network',
        text='Choose a network from the list below'
    )

    code, tag = radiolist.display()

    if code in (dialog.CANCEL, dialog.ESC) or not tag:
        return

    return [network['obj'] for network in properties if network['name'] == tag].pop()


def inventory_search_by_dns(agent, dialog, vm_search):
    """
    Search inventory for managed objects by their DNS name

    Args:
        agent (VConnector): A VConnector instance
        dialog    (Dialog): A Dialog instance
        vm_search   (bool): If True search for VMs only, otherwise
                            search for hosts only

    """
    datacenter = choose_datacenter(
        agent=agent,
        dialog=dialog,
        all_datacenters_option=True
    )

    code, name = dialog.inputbox(
        title='Inventory Search',
        text='Specify DNS name to search for'
    )

    if not name:
        dialog.msgbox(
            title='Error',
            text='Invalid input provided'
        )
        return

    dialog.infobox(
        text='Searching Inventory ...'
    )

    if datacenter:
        result = agent.si.content.searchIndex.FindAllByDnsName(
            datacenter=datacenter,
            dnsName=name,
            vmSearch=vm_search
        )
    else:
        result = agent.si.content.searchIndex.FindAllByDnsName(
            dnsName=name,
            vmSearch=vm_search
        )

    return result


def inventory_search_by_ip(agent, dialog, vm_search):
    """
    Search inventory for managed objects by their IP address

    Args:
        agent (VConnector): A VConnector instance
        dialog    (Dialog): A Dialog instance
        vm_search   (bool): If True search for VMs only, otherwise
                            search for hosts only

    """
    datacenter = choose_datacenter(
        agent=agent,
        dialog=dialog,
        all_datacenters_option=True
    )

    code, ipaddr = dialog.inputbox(
        title='Inventory Search',
        text='Specify IP address to search for'
    )

    if not ipaddr:
        dialog.msgbox(
            title='Error',
            text='Invalid input provided'
        )
        return

    dialog.infobox(
        text='Searching Inventory ...'
    )

    if datacenter:
        result = agent.si.content.searchIndex.FindAllByIp(
            datacenter=datacenter,
            ip=ipaddr,
            vmSearch=vm_search
        )
    else:
        result = agent.si.content.searchIndex.FindAllByIp(
            ip=ipaddr,
            vmSearch=vm_search
        )

    return result


def inventory_search_by_uuid(agent, dialog, vm_search):
    """
    Search inventory for managed objects by their UUID

    Args:
        agent (VConnector): A VConnector instance
        dialog    (Dialog): A Dialog instance
        vm_search   (bool): If True search for VMs only, otherwise
                            search for hosts only

    """
    datacenter = choose_datacenter(
        agent=agent,
        dialog=dialog,
        all_datacenters_option=True
    )

    code, uuid = dialog.inputbox(
        title='Inventory Search',
        text='Specify UUID to search for'
    )

    if not uuid:
        dialog.msgbox(
            title='Error',
            text='Invalid input provided'
        )
        return

    dialog.infobox(
        text='Searching Inventory ...'
    )

    if datacenter:
        result = agent.si.content.searchIndex.FindAllByUuid(
            datacenter=datacenter,
            uuid=uuid,
            vmSearch=vm_search
        )
    else:
        result = agent.si.content.searchIndex.FindAllByUuid(
            uuid=uuid,
            vmSearch=vm_search
        )

    return result



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Event Widgets

"""

import os
import time
import tempfile
import threading

import pyVmomi

__all__ = ['EventWidget', 'EventCollector']


class EventWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Event Widget

        Args:
            agent         (VConnector): A VConnector instance
            dialog     (dialog.Dialog): A Dialog instance
            obj    (vim.ManagedEntity): A Managed Entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        self.dialog.infobox(
            title=self.title,
            text='Retrieving information ...'
        )

        fd, path = tempfile.mkstemp(prefix='pvcevents-')

        # We need to create the destination file where events
        # will be appended to because we don't know exactly how long
        # the event history collector will take to put any new
        # data in the file, so the operation can take long enough
        # to create the file and dialog(1) needs a valid and
        # already existing file in order to display the widget
        with open(path, 'w'):
            pass

        self.dialog.infobox(
            title=self.title,
            text='Starting event collector ...'
        )

        collector = EventCollector(
            agent=self.agent,
            obj=self.obj,
            path=path
        )
        collector.start()

        # Give it some time before displaying the widget
        time.sleep(3)

        self.dialog.tailbox(
            filepath=path,
            title=self.title,
        )

        collector.signal_stop()
        collector.join(1)
        os.unlink(path)


class EventCollector(threading.Thread):
    def __init__(self, agent, obj, path):
        """
        Event Collector Thread

        Args:
            agent        (VConnector): A VConnector instance
            obj   (vim.ManagedEntity): A Managed Entity
            path                (str): Path where new events are appended to

        """
        super().__init__()
        self.daemon = True
        self.time_to_die = threading.Event()

        self.agent = agent
        self.obj = obj
        self.path = path
        self.last_event_key = 0

    def run(self):
        entity_filter_spec = pyVmomi.vim.event.EventFilterSpec.ByEntity(
            entity=self.obj,
            recursion=pyVmomi.vim.event.EventFilterSpec.RecursionOption.all
        )

        filter_spec = pyVmomi.vim.event.EventFilterSpec(
            disableFullMessage=False,
            entity=entity_filter_spec
        )

        collector = self.agent.si.content.eventManager.CreateCollectorForEvents(
            filter=filter_spec
        )

        while not self.time_to_die.is_set():
            latest_events = self.get_latest_events(collector)
            if latest_events:
                self.save_events(latest_events)
            time.sleep(1)

        collector.DestroyCollector()

    def signal_stop(self):
        """
        Signal the thread that it's time to die

        """
        self.time_to_die.set()

    def get_latest_events(self, collector):
        """
        Get latest events up to the last known event

        Args:
            collector (vim.event.EventHistoryCollector): A collector instance

        Returns:
            A list of the latest events

        """
        latest_events = []
        latest_page = collector.latestPage
        for e in latest_page:
            if e.key == self.last_event_key:
                break
            else:
                latest_events.append(e)

        # The last event is the one with the largest key
        self.last_event_key = max([e.key for e in latest_page])
        latest_events.sort(key=lambda x: x.key)

        return latest_events

    def save_events(self, events):
        """
        Append new events to a file

        Args:
            events (list): A list of vim.event.Event instances

        """
        with open(self.path, 'a') as f:
            for e in events:
                if e.userName:
                    f.write('[{}]: User {}: {}\n'.format(str(e.createdTime), e.userName, e.fullFormattedMessage))
                else:
                    f.write('[{}]: {}\n'.format(str(e.createdTime), e.fullFormattedMessage))



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Network Widget module

"""

import pvc.widget.debug
import pvc.widget.event
import pvc.widget.menu
import pvc.widget.form
import pvc.widget.common
import pvc.widget.virtualmachine

__all__ = ['NetworkWidget']


class NetworkWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Network Widget

        Args:
            agent     (VConnector): A VConnector instance
            dialog (dialog.Dialog): A Dialog instance
            obj      (vim.Network): A vim.Network managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Summary',
                description='General Information',
                on_select=self.summary
            ),
            pvc.widget.menu.MenuItem(
                tag='Virtual Machines',
                description='Virtual Machines using this network',
                on_select=pvc.widget.common.virtual_machine_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Hosts',
                description='Hosts using this network',
                on_select=pvc.widget.common.host_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Tasks',
                description='View Tasks',
            ),
            pvc.widget.menu.MenuItem(
                tag='Events',
                description='View Events',
                on_select=pvc.widget.event.EventWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Debug',
                description='Start a Python REPL console',
                on_select=pvc.widget.debug.DebugWidget,
                on_select_args=(locals(), globals())
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an item from the menu'
        )

        menu.display()

    def summary(self):
        """
        General information about the network

        """
        self.dialog.infobox(
            title=self.title,
            text='Retrieving information ...'
        )

        elements = [
            pvc.widget.form.FormElement(
                label='Name',
                item=self.obj.name
            ),
            pvc.widget.form.FormElement(
                label='Accessible',
                item=str(self.obj.summary.accessible)
            ),
            pvc.widget.form.FormElement(
                label='IP Pool',
                item=self.obj.summary.ipPoolName
            ),
        ]

        form = pvc.widget.form.Form(
            dialog=self.dialog,
            form_elements=elements,
            title=self.title,
            text='Network summary information'
        )

        form.display()



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Debug Widgets

"""

import code
import readline

__all__ = ['DebugWidget']


class DebugWidget(object):
    def __init__(self, l, g):
        """
        A debug widget that starts a REPL console

        Args:
            l (dict): Local namespace dictionary
            g (dict): Global namespace dictionary

        """
        self.namespace = dict(g)
        self.namespace.update(l)
        self.display()

    def display(self):
        code.InteractiveConsole(locals=self.namespace).interact()



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Administration module

"""

import pvc.widget.common
import pvc.widget.menu
import pvc.widget.motd

__all__ = ['AdministrationWidget']


class AdministrationWidget(object):
    def __init__(self, agent, dialog):
        """
        Administration widget

        Args:
            agent (VConnector): A VConnector instance
            dialog    (Dialog): A Dialog instance

        """
        self.agent = agent
        self.dialog = dialog
        self.title = 'Administration'
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Message',
                description='Message Of The Day',
                on_select=pvc.widget.motd.MOTDWidget,
                on_select_args=(self.agent, self.dialog)
            ),
            pvc.widget.menu.MenuItem(
                tag='Sessions',
                description='View Sessions',
                on_select=pvc.widget.common.session_menu,
                on_select_args=(self.agent, self.dialog)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an action to be performed'
        )

        menu.display()



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Inventory Widgets

"""

import pyVmomi

import pvc.widget.common
import pvc.widget.menu
import pvc.widget.datastore
import pvc.widget.hostsystem
import pvc.widget.network
import pvc.widget.radiolist
import pvc.widget.virtualmachine

__all__ = [
    'InventoryWidget', 'InventorySearchWidget',
    'InventorySearchHostWidget', 'InventorySearchVirtualMachineWidget',
    'InventoryDatacenterWidget',
]


class InventoryWidget(object):
    def __init__(self, agent, dialog):
        """
        Inventory widget

        Args:
            agent (VConnector): A VConnector instance
            dialog    (Dialog): A Dialog instance

        """
        self.agent = agent
        self.dialog = dialog
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Datacenters',
                description='Manage Datacenters',
                on_select=InventoryDatacenterWidget,
                on_select_args=(self.agent, self.dialog)
            ),
            pvc.widget.menu.MenuItem(
                tag='Clusters',
                description='Manage Clusters',
                on_select=pvc.widget.common.cluster_menu,
                on_select_args=(self.agent, self.dialog)
            ),
            pvc.widget.menu.MenuItem(
                tag='Hosts',
                description='Manage hosts',
                on_select=self.host_menu
            ),
            pvc.widget.menu.MenuItem(
                tag='VMs & Templates',
                description='Manage VMs & Templates',
                on_select=self.virtual_machine_menu
            ),
            pvc.widget.menu.MenuItem(
                tag='Datastores',
                description='Manage Datastores',
                on_select=self.datastore_menu
            ),
            pvc.widget.menu.MenuItem(
                tag='Networking',
                description='Manage Networking',
                on_select=self.network_menu
            ),
            pvc.widget.menu.MenuItem(
                tag='Search',
                description='Search Inventory',
                on_select=InventorySearchWidget,
                on_select_args=(self.agent, self.dialog)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Menu',
            text='Select an item from the inventory'
        )

        menu.display()

    def host_menu(self):
        self.dialog.infobox(
            text='Retrieving information ...'
        )

        view = self.agent.get_host_view()
        properties = self.agent.collect_properties(
            view_ref=view,
            obj_type=pyVmomi.vim.HostSystem,
            path_set=['name', 'runtime.connectionState'],
            include_mors=True
        )
        view.DestroyView()

        items = [
            pvc.widget.menu.MenuItem(
                tag=host['name'],
                description=host['runtime.connectionState'],
                on_select=pvc.widget.hostsystem.HostSystemWidget,
                on_select_args=(self.agent, self.dialog, host['obj'])
            ) for host in properties
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Hosts',
            text='Select a host from the menu'
        )

        menu.display()

    def datastore_menu(self):
        self.dialog.infobox(
            text='Retrieving information ...'
        )

        view = self.agent.get_datastore_view()
        properties = self.agent.collect_properties(
            view_ref=view,
            obj_type=pyVmomi.vim.Datastore,
            path_set=['name', 'summary.accessible'],
            include_mors=True
        )
        view.DestroyView()

        items = [
            pvc.widget.menu.MenuItem(
                tag=ds['name'],
                description='Accessible' if ds['summary.accessible'] else 'Not Accessible',
                on_select=pvc.widget.datastore.DatastoreWidget,
                on_select_args=(self.agent, self.dialog, ds['obj'])
            ) for ds in properties
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Datastores',
            text='Select a Datastore from the menu'
        )

        menu.display()

    def virtual_machine_menu(self):
        self.dialog.infobox(
            text='Retrieving information ...'
        )

        view = self.agent.get_vm_view()
        properties = self.agent.collect_properties(
            view_ref=view,
            obj_type=pyVmomi.vim.VirtualMachine,
            path_set=['name', 'runtime.powerState'],
            include_mors=True
        )
        view.DestroyView()

        items = [
            pvc.widget.menu.MenuItem(
                tag=vm['name'],
                description=vm['runtime.powerState'],
                on_select=pvc.widget.virtualmachine.VirtualMachineWidget,
                on_select_args=(self.agent, self.dialog, vm['obj'])
            ) for vm in properties
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Virtual Machines',
            text='Select a Virtual Machine from the menu'
        )

        menu.display()

    def network_menu(self):
        self.dialog.infobox(
            text='Retrieving information ...'
        )

        view = self.agent.get_container_view(
            obj_type=[pyVmomi.vim.Network]
        )

        properties = self.agent.collect_properties(
            view_ref=view,
            obj_type=pyVmomi.vim.Network,
            path_set=['name', 'summary.accessible'],
            include_mors=True
        )
        view.DestroyView()

        items = [
            pvc.widget.menu.MenuItem(
                tag=network['name'],
                description='Accessible' if network['summary.accessible'] else 'Not Accessible',
                on_select=pvc.widget.network.NetworkWidget,
                on_select_args=(self.agent, self.dialog, network['obj'])
            ) for network in properties
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Networks',
            text='Select a network from the menu that you wish to manage'
        )

        menu.display()


class InventorySearchWidget(object):
    def __init__(self, agent, dialog):
        """
        Inventory search widget

        Args:
            agent (VConnector): A VConnector instance
            dialog    (Dialog): A Dialog instance

        """
        self.agent = agent
        self.dialog = dialog
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Hosts',
                description='Search inventory for hosts',
                on_select=InventorySearchHostWidget,
                on_select_args=(self.agent, self.dialog)
            ),
            pvc.widget.menu.MenuItem(
                tag='Virtual Machines',
                description='Search inventory for VMs',
                on_select=InventorySearchVirtualMachineWidget,
                on_select_args=(self.agent, self.dialog)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Search',
            text=''
        )

        menu.display()


class InventorySearchHostWidget(object):
    def __init__(self, agent, dialog):
        """
        Widget to search inventory for hosts

        Args:
            agent (VConnector): A VConnector instance
            dialog    (Dialog): A Dialog instance

        """
        self.agent = agent
        self.dialog = dialog
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='DNS',
                description='Find hosts by DNS name',
                on_select=self.find_by_dns
            ),
            pvc.widget.menu.MenuItem(
                tag='IP',
                description='Find hosts by IP address',
                on_select=self.find_by_ip
            ),
            pvc.widget.menu.MenuItem(
                tag='UUID',
                description='Find hosts by UUID',
                on_select=self.find_by_uuid
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Search',
            text='Select criteria for searching of hosts'
        )

        menu.display()

    def find_by_dns(self):
        """
        Find hosts by their DNS name

        """
        result = pvc.widget.common.inventory_search_by_dns(
            agent=self.agent,
            dialog=self.dialog,
            vm_search=False
        )

        if not result:
            self.dialog.msgbox(
                title='Inventory Search',
                text='No results found'
            )
            return

        items = [
            pvc.widget.menu.MenuItem(
                tag=host.name,
                description=host.runtime.connectionState,
                on_select=pvc.widget.hostsystem.HostSystemWidget,
                on_select_args=(self.agent, self.dialog, host)
            ) for host in result
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Search Results',
            text='Found {} hosts matching the search criteria'.format(len(result))
        )

        menu.display()

    def find_by_ip(self):
        """
        Find hosts by their IP address

        """
        result = pvc.widget.common.inventory_search_by_ip(
            agent=self.agent,
            dialog=self.dialog,
            vm_search=False
        )

        if not result:
            self.dialog.msgbox(
                title='Inventory Search',
                text='No results found'
            )
            return

        items = [
            pvc.widget.menu.MenuItem(
                tag=host.name,
                description=host.runtime.connectionState,
                on_select=pvc.widget.hostsystem.HostSystemWidget,
                on_select_args=(self.agent, self.dialog, host)
            ) for host in result
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Search Results',
            text='Found {} hosts matching the search criteria'.format(len(result))
        )

        menu.display()

    def find_by_uuid(self):
        """
        Find hosts by their UUID

        """
        result = pvc.widget.common.inventory_search_by_uuid(
            agent=self.agent,
            dialog=self.dialog,
            vm_search=False
        )

        if not result:
            self.dialog.msgbox(
                title='Inventory Search',
                text='No results found'
            )
            return

        items = [
            pvc.widget.menu.MenuItem(
                tag=host.name,
                description=host.runtime.connectionState,
                on_select=pvc.widget.hostsystem.HostSystemWidget,
                on_select_args=(self.agent, self.dialog, host)
            ) for host in result
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Search Results',
            text='Found {} hosts matching the search criteria'.format(len(result))
        )

        menu.display()


class InventorySearchVirtualMachineWidget(object):
    def __init__(self, agent, dialog):
        """
        Widget to search inventory for virtual machines

        Args:
            agent (VConnector): A VConnector instance
            dialog    (Dialog): A Dialog instance

        """
        self.agent = agent
        self.dialog = dialog
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='DNS',
                description='Find VMs by DNS name',
                on_select=self.find_by_dns
            ),
            pvc.widget.menu.MenuItem(
                tag='IP',
                description='Find VMs by IP address',
                on_select=self.find_by_ip
            ),
            pvc.widget.menu.MenuItem(
                tag='UUID',
                description='Find VMs by UUID',
                on_select=self.find_by_uuid
            ),
            pvc.widget.menu.MenuItem(
                tag='Datastore Path',
                description='Find VM by its location on a datastore',
                on_select=self.find_by_datastore_path
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Search',
            text='Select criteria for searching of hosts'
        )

        menu.display()

    def find_by_dns(self):
        """
        Find virtual machines by their DNS name

        """
        result = pvc.widget.common.inventory_search_by_dns(
            agent=self.agent,
            dialog=self.dialog,
            vm_search=True
        )

        if not result:
            self.dialog.msgbox(
                title='Inventory Search',
                text='No results found'
            )
            return

        items = [
            pvc.widget.menu.MenuItem(
                tag=vm.name,
                description=vm.runtime.powerState,
                on_select=pvc.widget.virtualmachine.VirtualMachineWidget,
                on_select_args=(self.agent, self.dialog, vm)
            ) for vm in result
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Search Results',
            text='Found {} virtual machines matching the search criteria'.format(len(result))
        )

        menu.display()

    def find_by_ip(self):
        """
        Find virtual machines by their IP address

        """
        result = pvc.widget.common.inventory_search_by_ip(
            agent=self.agent,
            dialog=self.dialog,
            vm_search=True
        )

        if not result:
            self.dialog.msgbox(
                title='Inventory Search',
                text='No results found'
            )
            return

        items = [
            pvc.widget.menu.MenuItem(
                tag=vm.name,
                description=vm.runtime.powerState,
                on_select=pvc.widget.virtualmachine.VirtualMachineWidget,
                on_select_args=(self.agent, self.dialog, vm)
            ) for vm in result
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Search Results',
            text='Found {} virtual machines matching the search criteria'.format(len(result))
        )

        menu.display()

    def find_by_uuid(self):
        """
        Find virtual machines by their UUID

        """
        result = pvc.widget.common.inventory_search_by_uuid(
            agent=self.agent,
            dialog=self.dialog,
            vm_search=True
        )

        if not result:
            self.dialog.msgbox(
                title='Inventory Search',
                text='No results found'
            )
            return

        items = [
            pvc.widget.menu.MenuItem(
                tag=vm.name,
                description=vm.runtime.powerState,
                on_select=pvc.widget.virtualmachine.VirtualMachineWidget,
                on_select_args=(self.agent, self.dialog, vm)
            ) for vm in result
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Search Results',
            text='Found {} virtual machines matching the search criteria'.format(len(result))
        )

        menu.display()

    def find_by_datastore_path(self):
        """
        Find virtual machine it's localtion on a datastore

        """
        datacenter = pvc.widget.common.choose_datacenter(
            agent=self.agent,
            dialog=self.dialog,
            all_datacenters_option=False
        )

        if not datacenter:
            self.dialog.msgbox(
                title='Error',
                text='No datacenter specified for inventory search'
            )
            return

        code, path = self.dialog.inputbox(
            title='Inventory Search',
            text='Specify datastore path to the .vmx file'
        )

        if not path:
            self.dialog.msgbox(
                title='Error',
                text='Invalid input provided'
            )
            return

        self.dialog.infobox(
            text='Searching Inventory ...'
        )

        vm = self.agent.si.content.searchIndex.FindByDatastorePath(
            datacenter=datacenter,
            path=path
        )

        if not vm:
            self.dialog.msgbox(
                title='Inventory Search',
                text='No results found'
            )
            return

        items = [
            pvc.widget.menu.MenuItem(
                tag=vm.name,
                description=vm.runtime.powerState,
                on_select=pvc.widget.virtualmachine.VirtualMachineWidget,
                on_select_args=(self.agent, self.dialog, vm)
            )
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Search Results',
            text=''
        )

        menu.display()


class InventoryDatacenterWidget(object):
    def __init__(self, agent, dialog):
        """
        Inventory Datacenter Widget

        Args:
            agent (VConnector): A VConnector instance
            dialog    (Dialog): A Dialog instance

        """
        self.agent = agent
        self.dialog = dialog
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Create',
                description='Create new datacenter',
                on_select=self.create_datacenter
            ),
            pvc.widget.menu.MenuItem(
                tag='View',
                description='View datacenters',
                on_select=pvc.widget.common.datacenter_menu,
                on_select_args=(self.agent, self.dialog)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title='Inventory Datacenter Menu',
            text='Select an item from the inventory'
        )

        menu.display()

    def create_datacenter(self):
        """
        Create a new Datacenter

        """
        folder = pvc.widget.common.choose_folder(
            agent=self.agent,
            dialog=self.dialog
        )

        code, name = self.dialog.inputbox(
            title='Create Datacenter',
            text='Provide name for the Datacenter'
        )

        if code in (self.dialog.CANCEL, self.dialog.ESC):
            return

        if not name:
            self.dialog.msgbox(
                title='Error',
                text='Invalid input provided'
            )
            return

        self.dialog.infobox(
            text='Creating datacenter {} ...'.format(name)
        )

        try:
            folder.CreateDatacenter(name=name)
        except pyVmomi.vim.MethodFault as e:
            self.dialog.msgbox(
                title='Error',
                text=e.msg
            )



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Radio list widgets

"""

__all__ = ['RadioList', 'RadioListItem']


class RadioListItem(object):
    def __init__(self, tag, description='', status='off'):
        """
        A radio list item

        No more than one radio list item should be set to 'on'

        Args:
            tag         (str): Unique tag name for the item
            description (str): Short description of the item
            status      (str): Initial status of the item.
                               Using 'on' selects the item.
                               Using 'off' deselects the item.

        """
        self.tag = tag
        self.description = description
        self.status = status


class RadioList(object):
    def __init__(self, items, dialog, **kwargs):
        """
        Radio list class

        Args:
            items           (list): A list of RadioListItem instances
            dialog (dialog.Dialog): Dialog instance
            kwargs          (dict): Additional args to be passed to dialog(1)

        Returns:
            A tuple containing the code and selected tag

        """
        self.items = items
        self.dialog = dialog
        self.kwargs = kwargs
        self.choices = [(item.tag, item.description, item.status) for item in self.items]
        self._registry = {item.tag: item for item in self.items}

    def display(self):
        code, tag = self.dialog.radiolist(
            choices=self.choices,
            **self.kwargs
        )

        return (code, tag)



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Core Widgets

"""

import ssl

try:
    _create_unverified_https_context = ssl._create_unverified_context
except AttributeError:
    # Legacy Python that doesn't verify HTTPS certificates by default
    pass
else:
    # Handle target environment that doesn't support HTTPS verification
    ssl._create_default_https_context = _create_unverified_https_context

import pyVmomi

import requests
requests.packages.urllib3.disable_warnings()

import pvc.widget.form
import pvc.widget.home

from dialog import Dialog
from vconnector.core import VConnector

from pvc import __version__

__all__ = ['MainApp']


class MainApp(object):
    """
    Main App class

    """
    def __init__(self):
        self.dialog = Dialog(autowidgetsize=True)
        self.dialog.add_persistent_args(['--no-mouse'])
        self.dialog.set_background_title(
            'Python vSphere Client version {}'.format(__version__)
        )
        self.agent = None

    def about(self):
        welcome = (
            'Welcome to the Python vSphere Client version {}.\n\n'
            'PVC is hosted on Github. Please contribute by reporting '
            'issues, suggesting features and sending patches using '
            'pull requests.\n\n'
            'https://github.com/dnaeon/pvc'
        )

        self.dialog.msgbox(
            title='Welcome',
            text=welcome.format(__version__)
        )

    def login(self):
        """
        Login to the VMware vSphere host

        Returns:
            True on successful connect, False otherwise

        """
        form_text = (
            'Enter IP address or DNS name '
            'of the VMware vSphere host you wish '
            'to connect to.\n'
        )

        elements = [
            pvc.widget.form.FormElement(label='Hostname'),
            pvc.widget.form.FormElement(label='Username'),
            pvc.widget.form.FormElement(label='Password', attributes=0x1),
        ]

        form = pvc.widget.form.Form(
            dialog=self.dialog,
            form_elements=elements,
            mixed_form=True,
            title='Login Details',
            text=form_text,
        )

        while True:
            code, fields = form.display()
            if code in (self.dialog.CANCEL, self.dialog.ESC):
                return False

            if not all(fields.values()):
                self.dialog.msgbox(
                    title='Error',
                    text='Invalid login details, please try again.\n'
                )
                continue

            self.dialog.infobox(
                title='Establishing Connection',
                text='Connecting to {} ...'.format(fields['Hostname']),
            )

            self.agent = VConnector(
                host=fields['Hostname'],
                user=fields['Username'],
                pwd=fields['Password'],
            )

            try:
                self.agent.connect()
                text = '{} - {} - Python vSphere Client version {}'
                background_title = text.format(
                    self.agent.host,
                    self.agent.si.content.about.fullName,
                    __version__
                )
                self.dialog.set_background_title(background_title)
                return True
            except Exception as e:
                if isinstance(e, pyVmomi.vim.MethodFault):
                    msg = e.msg
                else:
                    msg = e

                self.dialog.msgbox(
                    title='Login failed',
                    text='Failed to login to {}\n\n{}\n'.format(self.agent.host, msg)
                )

    def disconnect(self):
        """
        Disconnect from the remote vSphere host

        """
        if not self.agent:
            return

        self.dialog.infobox(
            title='Disconnecting Connection',
            text='Disconnecting from {} ...'.format(self.agent.host)
        )
        self.agent.disconnect()

    def run(self):
        try:
            self.about()
            if not self.login():
                return

            home = pvc.widget.home.HomeWidget(
                agent=self.agent,
                dialog=self.dialog
            )
            home.display()
        except KeyboardInterrupt:
            pass
        finally:
            self.disconnect()



from __future__ import print_function

import re
import ast
import sys

from setuptools import setup, find_packages


_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('src/pvc/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1))
    )

if (2, 7) <= sys.version_info < (3, 0):
    dialog_package = 'python2-pythondialog'
elif sys.version_info >= (3, 0):
    dialog_package = 'pythondialog'
else:
    print('Unsupported Python version')
    sys.exit(1)

setup(
    name='pvc',
    version=version,
    description='Python vSphere Client with a dialog(1) interface',
    long_description=open('README.rst').read(),
    author='Marin Atanasov Nikolov',
    author_email='dnaeon@gmail.com',
    license='BSD',
    url='https://github.com/dnaeon/pvc',
    download_url='https://github.com/dnaeon/pvc/releases',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    scripts=[
        'src/pvc-tui',
    ],
    install_requires=[
        '{} >= 3.2.1'.format(dialog_package),
        'humanize >= 0.5.1',
        'pyvmomi >= 5.5.0-2014.1.1',
        'requests >= 2.6.0',
        'vconnector >= 0.3.7',
    ]
)



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
Datastore Widgets

"""

import humanize

import pvc.widget.alarm
import pvc.widget.common
import pvc.widget.debug
import pvc.widget.event
import pvc.widget.menu
import pvc.widget.form
import pvc.widget.gauge
import pvc.widget.performance
import pvc.widget.virtualmachine

__all__ = ['DatastoreWidget', 'DatastoreActionWidget']


class DatastoreWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Datastore Widget

        Args:
            agent     (VConnector): A VConnector instance
            dialog (dialog.Dialog): A Dialog instance
            obj    (vim.Datastore): A Datastore managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='General',
                description='General information',
                on_select=self.info
            ),
            pvc.widget.menu.MenuItem(
                tag='Capacity',
                description='Datastore Capacity ',
                on_select=self.capacity
            ),
            pvc.widget.menu.MenuItem(
                tag='Actions',
                description='Available Actions',
                on_select=DatastoreActionWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Hosts',
                description='Hosts using the datastore',
                on_select=pvc.widget.common.hostmount_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Virtual Machines',
                description='Virtual Machines using the datastore',
                on_select=pvc.widget.common.virtual_machine_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Performance',
                description='Performance Metrics',
                on_select=pvc.widget.performance.PerformanceProviderWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Events',
                description='View Events',
                on_select=pvc.widget.event.EventWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Alarms',
                description='View triggered alarms',
                on_select=pvc.widget.common.alarm_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Debug',
                description='Start a Python REPL console',
                on_select=pvc.widget.debug.DebugWidget,
                on_select_args=(locals(), globals())
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an action to be performed'
        )

        menu.display()

    def info(self):
        """
        Datastore general information

        """
        self.dialog.infobox(
            title=self.title,
            text='Retrieving information ...'
        )

        elements = [
            pvc.widget.form.FormElement(
                label='Name',
                item=self.obj.name,
            ),
            pvc.widget.form.FormElement(
                label='Location',
                item=self.obj.summary.url
            ),
            pvc.widget.form.FormElement(
                label='Type',
                item=self.obj.summary.type
            ),
            pvc.widget.form.FormElement(
                label='Accessible',
                item=str(self.obj.summary.accessible)
            ),
            pvc.widget.form.FormElement(
                label='Maintenance Mode',
                item=self.obj.summary.maintenanceMode
            ),
            pvc.widget.form.FormElement(
                label='Multiple Host Access',
                item=str(self.obj.summary.multipleHostAccess)
            ),
        ]

        form = pvc.widget.form.Form(
            dialog=self.dialog,
            form_elements=elements,
            title=self.title,
            text='Datastore general information'
        )

        form.display()

    def capacity(self):
        """
        Datastore Capacity Information

        """
        self.dialog.infobox(
            title=self.title,
            text='Retrieving information ...'
        )

        uncommitted = self.obj.summary.uncommitted if self.obj.summary.uncommitted else 0
        elements = [
            pvc.widget.form.FormElement(
                label='Capacity',
                item=humanize.naturalsize(self.obj.summary.capacity, binary=True)
            ),
            pvc.widget.form.FormElement(
                label='Free Space',
                item=humanize.naturalsize(self.obj.summary.freeSpace, binary=True)
            ),
            pvc.widget.form.FormElement(
                label='Uncommitted Space',
                item=humanize.naturalsize(uncommitted, binary=True)
            ),
        ]

        form = pvc.widget.form.Form(
            dialog=self.dialog,
            form_elements=elements,
            title=self.title,
            text='Datastore capacity information'
        )

        return form.display()


class DatastoreActionWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Datastore Actions Widget

        Args:
            agent     (VConnector): A VConnector instance
            dialog (dialog.Dialog): A Dialog instance
            obj    (vim.Datastore): A Datastore managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Rename',
                description='Rename datastore',
                on_select=pvc.widget.common.rename,
                on_select_args=(self.obj, self.dialog)
            ),
            pvc.widget.menu.MenuItem(
                tag='Refresh',
                description='Refresh storage information',
                on_select=self.refresh
            ),
            pvc.widget.menu.MenuItem(
                tag='Remove',
                description='Remove datastore',
                on_select=pvc.widget.common.remove,
                on_select_args=(self.obj, self.dialog)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an action to be performed'
        )

        menu.display()

    def refresh(self):
        """
        Refresh storage related information

        """
        self.dialog.infobox(
            title=self.title,
            text='Refreshing storage information ...'
        )

        self.obj.RefreshDatastoreStorageInfo()



# Copyright (c) 2015 Marin Atanasov Nikolov <dnaeon@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer
#    in this position and unchanged.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""
HostSystem Widgets

"""

import datetime

import pyVmomi
import humanize

import pvc.widget.alarm
import pvc.widget.checklist
import pvc.widget.common
import pvc.widget.debug
import pvc.widget.event
import pvc.widget.menu
import pvc.widget.form
import pvc.widget.network
import pvc.widget.performance
import pvc.widget.virtualmachine

__all__ = [
    'HostSystemWidget', 'HostSystemDatastoreWidget',
    'HostSystemAddNfsStorage', 'HostSystemUnmountStorage',
    'HostSystemVirtualMachineWidget', 'HostSystemServiceWidget',
]


class HostSystemWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        HostSystem Widget

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.HostSystem): A HostSystem managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='General',
                description='General information',
                on_select=self.info
            ),
            pvc.widget.menu.MenuItem(
                tag='Resources',
                description='Resource usage information',
                on_select=self.resources
            ),
            pvc.widget.menu.MenuItem(
                tag='Virtual Machines',
                description='Manage Virtual Machines',
                on_select=HostSystemVirtualMachineWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Networks',
                description='Host Networks',
                on_select=pvc.widget.common.network_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Datastores',
                description='Manage datastores on this host',
                on_select=HostSystemDatastoreWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Performance',
                description='Performance Metrics',
                on_select=pvc.widget.performance.PerformanceProviderWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Events',
                description='View Events',
                on_select=pvc.widget.event.EventWidget,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Alarms',
                description='View triggered alarms',
                on_select=pvc.widget.common.alarm_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Services',
                description='Manage Services',
                on_select=pvc.widget.common.host_service_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='Debug',
                description='Start a Python REPL console',
                on_select=pvc.widget.debug.DebugWidget,
                on_select_args=(locals(), globals())
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an action to be performed'
        )

        menu.display()

    def info(self):
        """
        General information

        """
        self.dialog.infobox(
            title=self.title,
            text='Retrieving information ...'
        )

        elements = [
            pvc.widget.form.FormElement(
                label='State',
                item=self.obj.runtime.connectionState
            ),
            pvc.widget.form.FormElement(
                label='Type',
                item=self.obj.config.product.fullName,
            ),
            pvc.widget.form.FormElement(
                label='Vendor',
                item=self.obj.hardware.systemInfo.vendor
            ),
            pvc.widget.form.FormElement(
                label='Model',
                item=self.obj.hardware.systemInfo.model
            ),
            pvc.widget.form.FormElement(
                label='Memory Size',
                item=humanize.naturalsize(self.obj.hardware.memorySize, binary=True)
            ),
            pvc.widget.form.FormElement(
                label='CPU Packages',
                item=str(self.obj.hardware.cpuInfo.numCpuPackages)
            ),
            pvc.widget.form.FormElement(
                label='CPU Cores',
                item=str(self.obj.hardware.cpuInfo.numCpuCores)
            ),
            pvc.widget.form.FormElement(
                label='CPU Threads',
                item=str(self.obj.hardware.cpuInfo.numCpuThreads)
            ),
            pvc.widget.form.FormElement(
                label='Uptime',
                item=str(datetime.timedelta(seconds=self.obj.summary.quickStats.uptime))
            ),
        ]

        form = pvc.widget.form.Form(
            dialog=self.dialog,
            form_elements=elements,
            title=self.title,
            text='Host general information'
        )

        form.display()

    def resources(self):
        """
        Resource usage information

        """
        self.dialog.infobox(
            title=self.title,
            text='Retrieving information ...'
        )

        elements = [
            pvc.widget.form.FormElement(
                label='CPU Usage',
                item='{} MHz'.format(self.obj.summary.quickStats.overallCpuUsage)
            ),
            pvc.widget.form.FormElement(
                label='Memory Usage',
                item='{} MB'.format(self.obj.summary.quickStats.overallMemoryUsage)
            ),
            pvc.widget.form.FormElement(
                label='CPU Fairness',
                item='{} MHz'.format(self.obj.summary.quickStats.distributedCpuFairness)
            ),
            pvc.widget.form.FormElement(
                label='Memory Fairness',
                item='{} MB'.format(self.obj.summary.quickStats.distributedMemoryFairness)
            ),
        ]

        form = pvc.widget.form.Form(
            dialog=self.dialog,
            form_elements=elements,
            title=self.title,
            text='Host resource usage information'
        )

        form.display()


class HostSystemVirtualMachineWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Widget for managing Virtual Machines on a HostSystem

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.HostSystem): A HostSystem managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Create',
                description='Create new Virtual Machine',
                on_select=pvc.widget.virtualmachine.CreateVirtualMachineWidget,
                on_select_args=(self.agent, self.dialog, self.obj.parent.parent.parent, self.obj.parent, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='View',
                description='View Virtual Machines on host',
                on_select=pvc.widget.common.virtual_machine_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an action to be performed'
        )

        menu.display()


class HostSystemDatastoreWidget(object):
    def __init__(self, agent, dialog, obj):
        """
        Widget for managing datastores on a HostSystem

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.HostSystem): A HostSystem managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Create',
                description='Create new datastore',
                on_select=self.create_datastore
            ),
            pvc.widget.menu.MenuItem(
                tag='Unmount',
                description='Unmount datastores',
                on_select=HostSystemUnmountStorage,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
            pvc.widget.menu.MenuItem(
                tag='View',
                description='View datastores',
                on_select=pvc.widget.common.datastore_menu,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an action to be performed'
        )

        menu.display()

    def create_datastore(self):
        """
        Create new datastore menu

        """
        items = [
            pvc.widget.menu.MenuItem(
                tag='NFS',
                description='Mount NFS volume',
                on_select=HostSystemAddNfsStorage,
                on_select_args=(self.agent, self.dialog, self.obj)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Add storage',
        )

        menu.display()


class HostSystemAddNfsStorage(object):
    def __init__(self, agent, dialog, obj):
        """
        Create new datastore using a NFS volume

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.HostSystem): A HostSystem managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        elements = [
            pvc.widget.form.FormElement(label='Server'),
            pvc.widget.form.FormElement(label='Path', item='/vol0/datastore'),
            pvc.widget.form.FormElement(label='Read-Only', item='False'),
            pvc.widget.form.FormElement(label='Datastore Name'),
        ]

        form = pvc.widget.form.Form(
            dialog=self.dialog,
            form_elements=elements,
            title=self.title,
            text='Provide details of the remote NFS volume'
        )

        code, fields = form.display()

        if code in (self.dialog.CANCEL, self.dialog.ESC):
            return

        if not all(fields.values()):
            self.dialog.msgbox(
                title=self.title,
                text='Error, invalid input provided'
            )
            return

        if fields['Read-Only'].lower() in ('yes', 'true'):
            access_mode = pyVmomi.vim.HostMountMode.readOnly
        else:
            access_mode = pyVmomi.vim.HostMountMode.readWrite

        self.dialog.infobox(
            title=self.title,
            text='Creating datastore {} ...'.format(fields['Datastore Name'])
        )

        spec = pyVmomi.vim.HostNasVolumeSpec(
            accessMode=access_mode,
            localPath=fields['Datastore Name'],
            remoteHost=fields['Server'],
            remotePath=fields['Path']
        )

        try:
            self.obj.configManager.datastoreSystem.CreateNasDatastore(
                spec=spec
            )
        except pyVmomi.vim.MethodFault as e:
            self.dialog.msgbox(
                title=self.title,
                text=e.msg
            )


class HostSystemUnmountStorage(object):
    def __init__(self, agent, dialog, obj):
        """
        Widget for unmount datastores from host

        Args:
            agent      (VConnector): A VConnector instance
            dialog  (dialog.Dialog): A Dialog instance
            obj    (vim.HostSystem): A HostSystem managed entity

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        self.dialog.infobox(
            title=self.title,
            text='Retrieving information ...'
        )

        if not self.obj.datastore:
            self.dialog.msgbox(
                title=self.title,
                text='There are no datastores mounted on this host'
            )
            return

        items = [
            pvc.widget.checklist.CheckListItem(
                tag=d.name,
                description='Accessible' if d.summary.accessible else 'Not Accessible',
            ) for d in self.obj.datastore
        ]

        checklist = pvc.widget.checklist.CheckList(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select datastore(s) to be unmounted'
        )

        checklist.display()
        selected = checklist.selected()

        if not selected:
            return

        text = (
            'The following datastore(s) will be unmounted.'
            '\n\n{}\n\n'
            'Unmount datastore(s) from host?\n'
        )

        code = self.dialog.yesno(
            title=self.title,
            text=text.format('\n'.join(selected))
        )

        if code in (self.dialog.ESC, self.dialog.CANCEL):
            return

        datastore_objects = [d for d in self.obj.datastore if d.name in selected]
        for datastore_obj in datastore_objects:
            self.dialog.infobox(
                title=self.title,
                text='Unmount datastore {} ...'.format(datastore_obj.name)
            )
            self.obj.configManager.datastoreSystem.RemoveDatastore(
                datastore=datastore_obj
            )


class HostSystemServiceWidget(object):
    def __init__(self, agent, dialog, obj, service):
        """
        Widget for managing services on a HostSystem

        Args:
            agent        (VConnector): A VConnector instance
            dialog    (dialog.Dialog): A Dialog instance
            obj      (vim.HostSystem): A HostSystem managed entity
            service (vim.HostService): A HostService instance

        """
        self.agent = agent
        self.dialog = dialog
        self.obj = obj
        self.service = service
        self.service_system = self.obj.configManager.serviceSystem
        self.title = '{} ({})'.format(self.obj.name, self.obj.__class__.__name__)
        self.display()

    def display(self):
        items = [
            pvc.widget.menu.MenuItem(
                tag='Info',
                description='View Service Info',
                on_select=self.info
            ),
            pvc.widget.menu.MenuItem(
                tag='Start',
                description='Start Service',
                on_select=self.start
            ),
            pvc.widget.menu.MenuItem(
                tag='Stop',
                description='Stop Service',
                on_select=self.stop
            ),
            pvc.widget.menu.MenuItem(
                tag='Restart',
                description='Restart Service',
                on_select=self.restart
            ),
            pvc.widget.menu.MenuItem(
                tag='Uninstall',
                description='Uninstall Service',
                on_select=self.uninstall
            ),
            pvc.widget.menu.MenuItem(
                tag='Policy',
                description='Update Service Policy',
                on_select=self.update_policy
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            title=self.title,
            text='Select an action to be performed'
        )

        menu.display()

    def info(self):
        """
        Display info about a service

        """
        elements = [
            pvc.widget.form.FormElement(
                label='Service',
                item=self.service.key
            ),
            pvc.widget.form.FormElement(
                label='Description',
                item=self.service.label
            ),
            pvc.widget.form.FormElement(
                label='Running',
                item=str(self.service.running)
            ),
            pvc.widget.form.FormElement(
                label='Policy',
                item=self.service.policy
            ),
            pvc.widget.form.FormElement(
                label='Required',
                item=str(self.service.required)
            ),
            pvc.widget.form.FormElement(
                label='Uninstallable',
                item=str(self.service.uninstallable)
            ),
        ]

        form = pvc.widget.form.Form(
            dialog=self.dialog,
            form_elements=elements,
            title=self.title,
            text='Service information'
        )

        form.display()

    def start(self):
        """
        Starts a service

        """
        self.dialog.infobox(
            title=self.title,
            text='Starting service {} ...'.format(self.service.label)
        )

        self.service_system.StartService(id=self.service.key)

    def stop(self):
        """
        Stops a service

        """
        self.dialog.infobox(
            title=self.title,
            text='Stopping service {} ...'.format(self.service.label)
        )

        self.service_system.StopService(id=self.service.key)

    def restart(self):
        """
        Restarts a service

        """
        self.dialog.infobox(
            title=self.title,
            text='Restarting service {} ...'.format(self.service.label)
        )

        self.service_system.RestartService(id=self.service.key)

    def uninstall(self):
        """"
        Uninstalls a service

        """
        self.dialog.infobox(
            title=self.title,
            text='Uninstall service {} ...'.format(self.service.label)
        )

        try:
            self.service_system.UninstallService(id=self.service.key)
        except pyVmomi.vim.MethodFault as e:
            self.dialog.msgbox(
                title=self.title,
                text=e.msg
            )

    def update_policy(self):
        """
        Updates the activation policy of a service

        """
        items = [
            pvc.widget.menu.MenuItem(
                tag='On',
                description='Enable service at boot-time',
                on_select=lambda x: x,
                on_select_args=(pyVmomi.vim.HostServicePolicy.on,)
            ),
            pvc.widget.menu.MenuItem(
                tag='Off',
                description='Disable service at boot-time',
                on_select=lambda x: x,
                on_select_args=(pyVmomi.vim.HostServicePolicy.off,)
            ),
            pvc.widget.menu.MenuItem(
                tag='Automatic',
                description='Start service only if it has open ports',
                on_select=lambda x: x,
                on_select_args=(pyVmomi.vim.HostServicePolicy.automatic,)
            ),
        ]

        menu = pvc.widget.menu.Menu(
            items=items,
            dialog=self.dialog,
            return_selected=True,
            title=self.title,
            text='Select service activation policy'
        )

        item = menu.display()

        if not isinstance(item, pvc.widget.menu.MenuItem):
            return

        policy = item.selected()

        self.dialog.infobox(
            title=self.title,
            text='Updating policy for service {} ...'.format(self.service.label)
        )

        self.service_system.UpdateServicePolicy(
            id=self.service.key,
            policy=policy
        )
